/**
 * @file gui_obj.h
 * @brief generic object implementation
 * @author Duboisset Philippe
 * @version 0.1b
 * @date (yyyy-mm-dd) 2013-07-13
 *
 * Copyright (C) <2013>  Duboisset Philippe <duboisset.philippe@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _gui_obj_h_
#define _gui_obj_h_

#include "gui.h"


typedef uint16_t signal_t;
typedef uint16_t state_t;
typedef uint8_t group_t;
typedef enum {
  E_RELEASED,
  E_RELEASED_TO_PUSHED,
  E_PUSHED,
  E_PUSHED_TO_RELEASED,
  E_MAX_EVENT
} event_e;
typedef void (*pObjFunc_t) (void *g_obj, void *obj);



/**
* @struct: g_obj_st
* @brief: generic object definition
*/
typedef struct g_obj_t {

  struct g_obj_t /*@null@*/ *next;  /*next generic object*/

  void *obj;                        /*physical object*/
  pObjFunc_t /*@null@*/ task;       /*task of the object; this one handles specific object refresh*/
  pObjFunc_t /*@null@*/ draw;       /*draw function of the object*/

  rect_st rec;                      /*absolute object window*/
  signal_t signals[E_MAX_EVENT];    /*signals table (one signal per possible event)*/
  state_t state;                    /*object state (disabled, hiden, focused, etc...)*/
  event_e event;                    /*user event (released, pushed, ...)*/
  group_t group;                    /*object group*/

} g_obj_st;

void GUI_SetGroup(group_t g);
void GUI_GroupDisable(group_t g, bool p);
void GUI_ReadTouchScreen(coord_t /*@out@*/ /*@null@*/ *px, coord_t /*@out@*/ /*@null@*/ *py);


void GUI_Init(void);
g_obj_st /*@null@*/ *GUI_AddGenericObject(void);
void GUI_ClearAll(void);
void GUI_SetSignal(event_e event, signal_t signal);
signal_t GUI_ReadSignal(void);
g_obj_st /*@null@*/ *GUI_GetLastAddedObject(void);
void GUI_DrawObjects(void);


bool GUI_ObjIsNotified(const g_obj_st /*@null@*/ *obj);
void GUI_ObjSetNotified(g_obj_st /*@null@*/ *obj, bool p);
bool GUI_ObjIsPressed(const g_obj_st /*@null@*/ *obj);
void GUI_ObjSetPressed(g_obj_st /*@null@*/ *obj, bool p);
bool GUI_ObjIsDisabled(const g_obj_st /*@null@*/ *obj);
void GUI_ObjSetDisabled(g_obj_st /*@null@*/ *obj, bool p);
bool GUI_ObjIsFocusable(const g_obj_st /*@null@*/ *obj);
void GUI_ObjSetFocusable(g_obj_st /*@null@*/ *obj, bool p);
bool GUI_ObjIsFocused(const g_obj_st /*@null@*/ *obj);
void GUI_ObjSetFocused(g_obj_st /*@null@*/ *obj, bool p);
bool GUI_ObjIsNeedRefresh(const g_obj_st /*@null@*/ *obj);
void GUI_ObjSetNeedRefresh(g_obj_st /*@null@*/ *obj, bool p);
void GUI_ObjSetBlink(g_obj_st /*@null@*/ *obj, bool p);
bool GUI_ObjIsBlink(const g_obj_st /*@null@*/ *obj);
void GUI_ObjSetStatic(g_obj_st /*@null@*/ *obj, bool p);
bool GUI_ObjIsStatic(const g_obj_st /*@null@*/ *obj);
#endif