/**
 * @file gui_demo_p1.c
 * @brief quick examples for GUI usage
 * @author Duboisset Philippe
 * @version 0.1b
 * @date (yyyy-mm-dd) 2013-07-13
 *
 * Copyright (C) <2013>  Duboisset Philippe <duboisset.philippe@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#include "gui_demo.h"

#define GRAPH_W               300
#define GRAPH_H               155
#define NB_MAX_CURVE          10
#define SIG_ADD_CURVE         1
#define SIG_CHECKBOX_CIRCULAR 2
#define SIG_BTN_NEXT          3


static uint16_t fps, frameCount;
static uint8_t wdtTrig;


static void WdtCallback(void) {
  if(frameCount > 0) {
    fps = frameCount;
    frameCount = 0;
    wdtTrig = 0;
  }
}


void DemoPage1(void) {

  uint8_t data[NB_MAX_CURVE][GRAPH_W];
  int16_t oldSample[NB_MAX_CURVE], ii, jj, nbCurve = 0;
  rect_st rec;
  g_obj_st *pGraph;
  uint8_t circularGraph = 0, done = 0;

  /*fps counter init. & graph samples init.*/
  frameCount = fps = 0;
  wdtTrig = 0;
  for(ii = 0; ii < NB_MAX_CURVE; ii++) {
    for(jj = 0; jj < GRAPH_W; jj++) {
      data[ii][jj] = ii * 20;
    }
    oldSample[ii] = ii * 20;
  }


  /*
   * Page construction
   */

  /*delete all objects & clear the screen*/
  GUI_ClearAll();

  /*title*/
  rec.x = 5; rec.y = 0; rec.w = 310; rec.h = 215;
  SetFont(G_FONT_BOLD);
  GUI_AddFrame(&rec, "Graph (random & scrolled)");
  SetFont(G_FONT_DEFAULT);

  /*parent graph (empty, add curves later)*/
  rec.x = 10; rec.y = 25; rec.w = GRAPH_W; rec.h = GRAPH_H;
  pGraph = GUI_AddGraph(&rec, 0 /*refresh time: 0ms <=> draw every software cycle*/ );

  /*FPS box*/
  rec.y += rec.h + 8; rec.w = 55; rec.h = 20;
  GUI_AddValueBox(&rec, &fps, BOX_T_UINT16, "%d fps");
  
  /*nb of curve box*/
  rec.x += rec.w + 5; rec.w = 105;
  GUI_AddValueBox(&rec, &nbCurve, BOX_T_UINT16, "Curve count: %d");

  /*button 'add curve' */
  rec.x += rec.w + 5; rec.w = 50;
  GUI_AddButton(&rec, "Add", 3 /*3 <=> image <green plus>*/);
  GUI_SetSignal(E_PUSHED_TO_RELEASED, SIG_ADD_CURVE);

  /*checkbox 'scroll graph'*/
  rec.x += rec.w + 5; rec.w = 71;
  GUI_AddCheckBox(&rec, "Circular", &circularGraph);
  GUI_SetSignal(E_PUSHED_TO_RELEASED, SIG_CHECKBOX_CIRCULAR);

  /*button 'next' */
  rec.x = 265; rec.y = 218; rec.w = 50;
  GUI_AddButton(&rec, "Next", 5 /*5 <=> image <next>*/);
  GUI_SetSignal(E_PUSHED_TO_RELEASED, SIG_BTN_NEXT);

  /*button 'previous' (disabled, no previous page)*/
  rec.x = 210;
  GUI_AddButton(&rec, "Prev.", 6 /*6 <=> image <previous>*/);
  GUI_ObjSetDisabled(NULL, true);

  /*
   * main loop
   */
  while(done == 0) {

    /**
     * user process
     */
    /*modify samples data*/
    for(ii = 0; ii < nbCurve; ii++) {

      /*normal graph: recompute all samples*/
      if(circularGraph == 0) {
        for(jj = 0; jj < GRAPH_W; jj++) {
          oldSample[ii] += P2D_Rand(10) - 5;
          if(oldSample[ii] < 0) oldSample[ii] = 0;
          else if(oldSample[ii] > 255) oldSample[ii] = 255;
          data[ii][jj] = oldSample[ii];
        }
      }
      /*circular (~~scrolled) graph: just add some samples*/
      else {
        for(jj = 0; jj < 1 /*number of samples to add*/; jj++) {
          oldSample[ii] += P2D_Rand(10) - 5;
          if(oldSample[ii] < 0) oldSample[ii] = 0;
          else if(oldSample[ii] > 255) oldSample[ii] = 255;
          GUI_AddSampleToCurve(pGraph, ii, oldSample[ii]);
        }
      }
    }

    /*update fps count*/
    frameCount++;
    if(wdtTrig == 0) {
      wdtTrig = 1;
      TicksSetWatchdog(WdtCallback, 1000);
    }


    /**
     * GUI process
     */
    /*execute GUI task*/
    GUI_DrawObjects();

    /*handle user events*/
    switch(GUI_ReadSignal()) {

      case SIG_ADD_CURVE:
        if(nbCurve < NB_MAX_CURVE) {
          GUI_AddCurveToGraph(pGraph, data[nbCurve], P2D_Rand(0xFFFF));
          nbCurve++;
        }
        break;

      case SIG_CHECKBOX_CIRCULAR:
        circularGraph = !circularGraph;
        break;

      case SIG_BTN_NEXT:
        SetPage(DemoPage2);
        done = 1;
        break;
    }
  }
}
