/**
 * @file gui_demo_p3.c
 * @brief quick examples for GUI usage
 * @author Duboisset Philippe
 * @version 0.1b
 * @date (yyyy-mm-dd) 2013-07-13
 *
 * Copyright (C) <2013>  Duboisset Philippe <duboisset.philippe@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "gui_demo.h"

#define SIG_BTN_PREV          1
#define SIG_BTN_NEXT          2
#define SIG_DISABLE_OBJ       3
#define SIG_NOTIFY_OBJ        4


void DemoPage3(void) {

  rect_st rec;
  float fVal;
  uint8_t u8Slider = 128;
  uint16_t u16Val;
  uint32_t u32Val;
  uint8_t done = 0, notifyObj = 0, disableObj = 0;

  g_obj_st *pSlider, *pNotifiedBox, *pDisabledBox;

  /*
   * Page construction
   */

  /*delete all objects & clear the screen*/
  GUI_ClearAll();

  /*title*/
  rec = GUI_Rect(5, 0, 310, 215);
  SetFont(G_FONT_BOLD);
  GUI_AddFrame(&rec, "Slider & value box");
  SetFont(G_FONT_DEFAULT);

  /*controls*/
  rec = GUI_Rect(10, 190, 110, 19);
  GUI_AddCheckBox(&rec, "Notify objects", &notifyObj);
  GUI_SetSignal(E_PUSHED_TO_RELEASED, SIG_NOTIFY_OBJ);

  rec = GUI_Rect(130, 190, 115, 19);
  GUI_AddCheckBox(&rec, "Disable objects", &disableObj);
  GUI_SetSignal(E_PUSHED_TO_RELEASED, SIG_DISABLE_OBJ);

  /*button 'next' */
  rec = GUI_Rect(265, 218, 50, 20);
  GUI_AddButton(&rec, "Next", 5 /*5 <=> image <next>*/);
  GUI_SetSignal(E_PUSHED_TO_RELEASED, SIG_BTN_NEXT);

  /*button 'previous' (disabled, no previous page)*/
  rec = GUI_Rect(210, 218, 50, 20);
  GUI_AddButton(&rec, "Prev.", 6 /*6 <=> image <previous>*/);
  GUI_SetSignal(E_PUSHED_TO_RELEASED, SIG_BTN_PREV);


  rec = GUI_Rect(10, 20, 220, 19);
  GUI_AddText(&rec, "Default value box, (e.g. float):");
  rec = GUI_Rect(230, 20, 75, 19);
  GUI_AddValueBox(&rec, &fVal, BOX_T_FLOAT, "%.3f");

  rec = GUI_Rect(10, 42, 220, 19);
  GUI_AddText(&rec, "Value box with format string:");
  rec = GUI_Rect(230, 42, 75, 19);
  GUI_AddValueBox(&rec, &fVal, BOX_T_FLOAT, "I=%.2f A");

  rec = GUI_Rect(10, 70, 300, 19);
  GUI_AddText(&rec, "Value boxes of different type, format & style:");
  rec = GUI_Rect(10, 92, 55, 19);
  GUI_AddValueBox(&rec, &u8Slider, BOX_T_INT8, "i8:%d");
  rec = GUI_Rect(75, 92, 55, 19);
  GUI_AddValueBox(&rec, &u8Slider, BOX_T_UINT8, "u8:%d");
  rec = GUI_Rect(140, 92, 75, 19);
  GUI_AddValueBox(&rec, &u16Val, BOX_T_INT16, "i16:%d");
  rec = GUI_Rect(225, 92, 75, 19);
  GUI_AddValueBox(&rec, &u16Val, BOX_T_UINT16, "u16:%d");

  rec = GUI_Rect(10, 114, 110, 19);
  GUI_AddValueBox(&rec, &u32Val, BOX_T_INT32, "i32:%d");

  SetColor(G_COL_TEXT, P2D_Color(255, 0, 0));
  rec = GUI_Rect(125, 114, 65, 19);
  GUI_AddValueBox(&rec, &u16Val, BOX_T_UINT16, "0x%04X");
  SetColor(G_COL_TEXT, P2D_Color(0, 0, 0));
  
  SetFont(G_FONT_DIGIT);
  rec = GUI_Rect(195, 114, 65, 19);
  GUI_AddValueBox(&rec, &u16Val, BOX_T_UINT16, "%04d");
  SetFont(G_FONT_DEFAULT);


  rec = GUI_Rect(10, 136, 145, 19);
  pDisabledBox = GUI_AddValueBox(&rec, &u8Slider, BOX_T_UINT8, "Disable me! u8:%d");
  rec = GUI_Rect(160, 136, 145, 19);
  pNotifiedBox = GUI_AddValueBox(&rec, &u8Slider, BOX_T_UINT8, "Notify me! u8:%d");


  rec = GUI_Rect(20, 165, 280, 15);
  pSlider = GUI_AddSlider(&rec, &u8Slider);

  /*main loop*/
  while(done == 0) {

    /**
     * user process
     */
     fVal = (float) u8Slider / 255;
     u16Val = fVal * 0xFFFF;
     u32Val = fVal * 0xFFFFFFFF;


    /**
     * GUI process
     */
    /*execute GUI task*/
    GUI_DrawObjects();

    /*handle user events*/
    switch(GUI_ReadSignal()) {

      case SIG_NOTIFY_OBJ:
        notifyObj = !notifyObj;
        GUI_ObjSetNotified(pSlider, notifyObj);
        GUI_ObjSetNotified(pNotifiedBox, notifyObj);
        break;
        
      case SIG_DISABLE_OBJ:
        disableObj = !disableObj;
        GUI_ObjSetDisabled(pSlider, disableObj);
        GUI_ObjSetDisabled(pDisabledBox, disableObj);
        break;

      case SIG_BTN_NEXT:
        SetPage(DemoPage4);
        done = 1;
        break;

      case SIG_BTN_PREV:
        SetPage(DemoPage2);
        done = 1;
        break;
    }
    DelayMs(1);
  }
}
