/**
 * @file gui_demo_p4.c
 * @brief quick examples for GUI usage
 * @author Duboisset Philippe
 * @version 0.1b
 * @date (yyyy-mm-dd) 2013-07-13
 *
 * Copyright (C) <2013>  Duboisset Philippe <duboisset.philippe@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#include "gui_demo.h"

#define SIG_BTN_PREV          1
#define SIG_BTN_NEXT          2

void DemoPage4(void) {

  rect_st rec;
  coord_t xMin = -4, xMax = 320, yMin = 18, yMax = 205;
  length_t w = 15, h = 15;
  uint16_t imgId = 0, cnt = 0, ms = 0;
  uint8_t done = 0;

  timer_t tStart;

  /*
   * Page construction
   */

  tStart = TicksGet();

  /*delete all objects & clear the screen*/
  GUI_ClearAll();

  rec = GUI_Rect(5, 0, 310, 215);
  SetFont(G_FONT_BOLD);
  GUI_AddFrame(&rec, "Massive objects rendering");
  SetFont(G_FONT_DEFAULT);

  rec.h = h;
  rec.w = w;
  for(rec.y = yMin; rec.y < yMax; rec.y += h) {
    for(rec.x = xMin; rec.x < xMax; rec.x += w) {
      if(GUI_AddButton(&rec, NULL, imgId++ % 44 + 1) != NULL) cnt++;
    }
  }


  rec = GUI_Rect(5, 218, 120, 20);
  GUI_AddValueBox(&rec, &cnt, BOX_T_UINT16, "Object count: %d");

  rec = GUI_Rect(125, 218, 80, 20);
  GUI_AddValueBox(&rec, &ms, BOX_T_UINT16, "...in %d ms");

  /*button 'next' */
  rec = GUI_Rect(265, 218, 50, 20);
  GUI_AddButton(&rec, "Next", 5 /*5 <=> image <next>*/);
  GUI_SetSignal(E_PUSHED_TO_RELEASED, SIG_BTN_NEXT);

  /*button 'previous'*/
  rec = GUI_Rect(210, 218, 50, 20);
  GUI_AddButton(&rec, "Prev.", 6 /*6 <=> image <previous>*/);
  GUI_SetSignal(E_PUSHED_TO_RELEASED, SIG_BTN_PREV);

  /*
   * main loop
   */
  while(done == 0) {

    /**
     * user process
     */
     /*none*/


    /**
     * GUI process
     */
    /*execute GUI task*/
    GUI_DrawObjects();

    /*handle user events*/
    switch(GUI_ReadSignal()) {
      case SIG_BTN_NEXT:
        SetPage(PageDigitalIO);
        done = 1;
        break;

      case SIG_BTN_PREV:
        SetPage(DemoPage3);
        done = 1;
        break;
    }

    if(ms == 0) {
      ms = TicksGet() - tStart;
    }

    DelayMs(1);
  }

}